/*
    *
    * This file is a part of CoreArchiver.
    * Archive manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/


#include <QFileDialog>
#include <QtConcurrent>

#include <cprime/infofunc.h>
#include <libarchiveqt.h>

#include "corearchiver.h"
#include "ui_corearchiver.h"


corearchiver::corearchiver(QWidget *parent) : QWidget(parent), ui(new Ui::corearchiver)
{
    ui->setupUi(this);

    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, Qt::transparent);
    setPalette(pltt);

    startSetup();
}

corearchiver::~corearchiver()
{
    delete ui;
}

void corearchiver::startSetup()
{
    connect(ui->nameLE, SIGNAL(textChanged(QString)), this, SLOT(updateFileName(QString)));
    connect(ui->locationTB, SIGNAL(clicked()), this, SLOT(updateDirName()));

    //add the archive formate type
    ui->formatsCB->addItems(QStringList() << ".zip" << ".tar.xz" << ".tar.gz" << ".tar.bz2" << ".7z" <<
                            ".tar" << ".iso" << ".cpio" << ".shar" << ".tar.Z" << ".ar" << ".xar");
    connect(ui->formatsCB, SIGNAL(currentIndexChanged(QString)), this, SLOT(updateFormat(QString)));

    connect(ui->createArc, &QToolButton::clicked, this, &corearchiver::doAction);
    connect(ui->cancel, SIGNAL(clicked()), this, SLOT(close()));

    ui->locationLE->setEnabled(false);
    format = ui->formatsCB->currentText();
    ui->nameLE->setText(archiveName);
    ui->nameLE->setFocus();
}

void corearchiver::doAction()
{
    if (extractF && !archiveF) {
        if (filePath.count() && location.count()) {
            extract(filePath, QDir(location));
        }
    } else if (!extractF && archiveF) {
        if (filePathList.count() && workingDir.count()) {
            QDir d(workingDir);
            compress(filePathList, d);
        }
    } else {
        qDebug() << "Unstable condition...";
    }
}

/*
 * Info :
 *
 * location = where the compressed file will be located
 * working dir = where the file is taken from
 *
 *
*/

void corearchiver::setFilename(const QString &fileName)
{
    archiveName = fileName;
    ui->nameLE->setText(archiveName);
}

void corearchiver::setFolderPath(const QString &path)
{
    workingDir = path;
    location = path;
}

void corearchiver::compress(const QStringList &archiveList, const QDir &currentDir)
{
    ui->nameLE->setEnabled(false);
    ui->locationLE->setEnabled(false);
    ui->locationTB->setEnabled(false);
    ui->formatsCB->setEnabled(false);
    ui->createArc->setEnabled(false);
    ui->cancel->setEnabled(false);

    QDir::setCurrent(currentDir.path());

    if (archiveName.isEmpty()) {
        return;
    }

    QString name = archiveName;

    if (!archiveName.endsWith(format)) {
        name += format;
    }

    QString fullArchivePath = QDir(location).filePath(name);
    LibArchiveQt *arc = new LibArchiveQt(fullArchivePath);

    if (arc == nullptr) {
        return;
    }

    arc->setWorkingDir(currentDir.currentPath());

    QStringList inputList;

    Q_FOREACH (QString file, archiveList) {
        QString newFn = file.replace(workingDir, "");

        if (newFn.startsWith("/")) {
            newFn.remove(0, 1);
        }

        inputList.append(newFn);
    }

    arc->updateInputFiles(inputList);

    QObject::connect(arc, &LibArchiveQt::jobComplete, this, &corearchiver::showArchiveMessage);

    arc->createArchive();
}

/**
 * @brief coreArc::extract Extract the archive at the archive path
 * @param archive Archive file path
 */
void corearchiver::extract(const QString &archiveFilePath, const QDir &dest)
{
    const QString destP = dest.path();

    // Create the dest folder if it does nor exist
    if (!QDir(destP).exists()) {
        QDir::current().mkdir(destP);
    }

    LibArchiveQt *arc = new LibArchiveQt(archiveFilePath);
    arc->setDestination(destP);

    QObject::connect(arc, &LibArchiveQt::jobComplete, this, &corearchiver::showExtractMessage);

    arc->extractArchive();
}

void corearchiver::updateFileName(const QString &fn)
{
    archiveName = QString(fn);
}

void corearchiver::updateDirName()
{
    const QString loc = QFileDialog::getExistingDirectory(this, "Choose Directory", QDir::currentPath());

    if (not loc.isEmpty()) {
        ui->locationLE->setEnabled(true);
        ui->locationLE->setText(loc);
        location = loc;
    }
}

void corearchiver::updateFormat(const QString &fmt)
{
    format = QString(fmt);
}

void corearchiver::sendFiles(const QStringList &paths, bool archive)
{
    if (!paths.count()) {
        return;
    }

    if (archive) {
        ui->createArc->setText("Create Archive");

        extractF = 0;
        archiveF = 1;
        filePath = "";

        filePathList = paths;

        if (paths.count() > 1) {
            setFilename(QFileInfo(paths.at(0)).dir().dirName());
        } else {
            setFilename(QFileInfo(paths.at(0)).fileName());
        }

        setFolderPath(QFileInfo(paths.at(0)).path());
        //        compress(paths, QDir(paths.at(0)));
    } else {
        if (paths.count() > 1) {
            qDebug() << "Sorry Multiple Extraction not supported. Only top file will be extracted.";
        }

        qDebug() << "paths " << paths;
        QFileInfo fi(paths.at(0));
        ui->locationLE->setText(fi.path());
        ui->nameLE->setEnabled(0);
        ui->createArc->setText("Extract");
        ui->format->setVisible(0);
        ui->formatsCB->setVisible(0);

        extractF = 1;
        archiveF = 0;
        filePath = fi.absoluteFilePath();

        setFilename(fi.fileName());
        setFolderPath(fi.path());
        //extract(paths.at(0), QDir(paths.at(0)).path());
    }
}

void corearchiver::showArchiveMessage()
{
    hide();
    CPrime::InfoFunc::messageEngine("CoreArchiver", "corearchiver", "Archived Successfully", "No issues, everthing went smoothly", this);
    QTimer::singleShot(5000, this, SLOT(close()));
}

void corearchiver::showExtractMessage()
{
    hide();
    CPrime::InfoFunc::messageEngine("CoreArchiver", "corearchiver", "Extracted Successfully...", "No issues, everthing went smoothly", this);
    QTimer::singleShot(5000, this, SLOT(close()));
}
